/*
        Programma in grado di convertire gli accenti tra due diverse tavole
        ascii estese. Inoltre converte in ascii standard (accentando con ')
        o in formato "stile html" (in questo caso converte anche &).
        Le lettere accentate prese in considerazione sono , , , , , .
        Inoltre sono convertiti i simboli , ,  e (non sempre) .
*/

#include <stdio.h>
#include <string.h>

enum { ASCII, HTML, DOS, WIN, INFORM } style;

int main(int argc, char** argv) {
    int i;
    char nf[128];
    FILE* h;

    *nf=0; style=ASCII;
    for (i=1;i<argc;i++) {
        if (argv[i][0]=='-') {
            if (options(argv[i])) return(0);
        } else {
            if (*nf==0) strncpy(nf,argv[i],127);
        }
    }

    if (*nf==0) h=stdin;
     else       if ((h=fopen(nf,"rt"))==NULL) return(-1);

    switch (style) {
        case WIN: winlatin_convert(h); break;
        case DOS: tab850_convert(h); break;
        case HTML: html_convert(h); break;
        case ASCII: ascii_convert(h); break;
        case INFORM: inform_convert(h); break;
        default: no_convert(h); break;
    }

    if (*nf!=0) fclose(h);
    return(0);
}

int ascii_convert(FILE *h) {
    char c;
    for (;;) {
        c=fgetc(h);
        if (feof(h)) break;
        switch(c) {
            case '': case '': printf("a'"); break;
            case '': case '':
            case '': case '': printf("e'"); break;
            case '': case '': printf("i'"); break;
            case '': case '': printf("o'"); break;
            case '': case '': printf("u'"); break;
            case '': case '': printf("c"); break;
            case '': case '': printf("^"); break;
            case '': case '': printf("|"); break;
            case '':           printf("L"); break;
            default: putc(c,stdout);
        }
    }
}

int tab850_convert(FILE *h) {
    char c;
    for (;;) {
        c=fgetc(h);
        if (feof(h)) break;
        switch(c) {
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            default: putc(c,stdout);
        }
    }
}

int winlatin_convert(FILE *h) {
    char c;
    for (;;) {
        c=fgetc(h);
        if (feof(h)) break;
        switch(c) {
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            case '': fputc('',stdout); break;
            default: putc(c,stdout);
        }
    }
}

int html_convert(FILE *h) {
    char c;
    for (;;) {
        c=fgetc(h);
        if (feof(h)) break;
        switch(c) {
            case '': case '': printf("&agrave;"); break;
            case '': case '': printf("&egrave;"); break;
            case '': case '': printf("&eacute;"); break;
            case '': case '': printf("&igrave;"); break;
            case '': case '': printf("&ograve;"); break;
            case '': case '': printf("&ugrave;"); break;
            case '': case '': printf("&ccedil;"); break;
            case '': case '': printf("&deg;"); break;
            case '': case '': printf("&sect;"); break;
                      case '': printf("&pound;"); break;
            case '&':           printf("&amp;");
            default: putc(c,stdout);
        }
    }
}

int inform_convert(FILE *h) {
    char c;
    for (;;) {
        c=fgetc(h);
        if (feof(h)) break;
        switch(c) {
            case '': case '': printf("@`a"); break;
            case '': case '': printf("@`e"); break;
            case '': case '': printf("@'e"); break;
            case '': case '': printf("@`i"); break;
            case '': case '': printf("@`o"); break;
            case '': case '': printf("@`u"); break;
            default: putc(c,stdout);
        }
    }
}

int no_convert(FILE *h) {
    char c;
    for (;;) {
        c=fgetc(h);
        if (feof(h)) break;
        putc(c,stdout);
    }
}


int options(char* s) {
    char opt_c[]="hvltidw";
    char opt_s[][16]={"help", "version", "license", "html", "inform", "dos", "win" };
    int i,xit=0,k;

    if (*s!='-') return(0);
    if (s[1]=='-') {
        for(i=0;i<strlen(opt_c);i++) {
            if (!strcmp(opt_s[i],s+2)) return(do_option(opt_c[i]));
        }
        fprintf(stderr,"Flag unknown: %s\n",s);
    } else {
        for (i=1;i<strlen(s);i++) {
            if ((strchr(opt_c,s[i])!=NULL)) {
                k=do_option(s[i]);
                xit=xit || k;
            } else {
                fprintf(stderr,"Flag unknown: -%c\n",s[i]);
            }
        }
        return(xit);
    }
}

int do_option(int o) {
    switch(o) {
        case 'h' : help(); return(1); break;
        case 'v' : version(); return(1); break;
        case 'l' : license(); return(1); break;
        case 't' : style=HTML; break;
        case 'i' : style=INFORM; break;
        case 'd' : style=DOS; break;
        case 'w' : style=WIN; break;
    }
    return(0);
}

int help() {
    printf("Accentuation converter v 1.1 by Paolo Lucchesi. Usage:\n");
    printf("accenti [-options] [filename]\n");
    printf("default: output in standard ascii\n");
    printf("-t or --html    Output in html-style\n");
    printf("-i or --inform  Output in inform-style\n");
    printf("-d or --dos     Output in dos codepage 850 extended ascii\n");
    printf("-w or --win     Output in windows latin extended ascii\n");
    printf("-h or --help    Display this page\n");
    printf("-v or --version Display version number\n");
    printf("-l or --license Display software license\n");
    return(0);
}

int license() {
    printf("Accentuation converter v 1.1 by Paolo Lucchesi.\n");
    printf("This program is freeware. Any user can freely use,\n");
    printf("distribute and modify this software without any restriction\n");
    printf("This software is distributed AS IS, with no warranty.\n");
    printf("The author is not responsible for any damage from the use\n");
    printf("of this software.\n");
    return(0);
}

int version() {
    printf("Accentuation converter v 1.1\n");
    return(0);
}

