/*
    pnginfo : a png info viewer by Paolo Lucchesi
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

struct {
    unsigned long width;
    unsigned long height;
    unsigned char bitdepth;
    unsigned char colortype;
    unsigned char compression;
    unsigned char filter;
    unsigned char interlace;
} typedef imagedata;

imagedata image;

int chunklist=0;

int main(int argc, char** argv) {
    int i;
    char nf[128];
    FILE* h;
    char s[128];

    *nf=0;
    for (i=1;i<argc;i++) {
        if (argv[i][0]=='-') {
            if (options(argv[i])) return(0);
        } else {
            if (*nf==0) strncpy(nf,argv[i],127);
        }
    }
    if (*nf==0) { shorthelp(); return(0); }
    pnginfo(nf);
    return(0);
}

int pnginfo(char* filename) {
    FILE *in;
    void* p;
    long dlen,i,j,os;
    int res;

    printf("Reading file %s\n",filename);
    if ((in=fopen(filename,"rb"))==NULL) {
        printf("File Error!\n"); return(-1); }

    if (check_signature(in)) {
        printf("Not a png file!\n"); return(-1); }

    if (read_chunk(in)) {
        fclose(in); printf("Header not found!\n"); return(-1); }

    if (chunklist) while (!read_chunk(in));

    fclose(in);
    return(0);
}

int check_signature(FILE* h) {
    int i,r=0;
    int sig[]={137, 80, 78, 71, 13, 10, 26, 10, -1};

    for (i=0;sig[i]>=0;i++) {
        if (fgetc(h)!=sig[i]) r=-1;
        if (feof(h)) {r=-1; break; }
    }
    return(r);
}

long ngetlong(FILE* h) {
    long r=0;
    int i;
    for (i=0;i<4;i++) r=r*256+fgetc(h);
    return(r);
}

int read_chunk(FILE* h) {
    unsigned long len,crc;
    char type[5];
    long i;

    len=ngetlong(h);
    fread(type,1,4,h); type[4]=0;
    do {
        if (!strnicmp(type,"IHDR",4) && len==13) { read_header(h); break; }

        fseek(h,len,SEEK_CUR);
        crc=ngetlong(h);
        printf("Chunk %s    len=%6d       crc=%08X\n",type,len,crc);
    } while(0);
    if (feof(h)) return(-1);
    if (!strnicmp(type,"IEND",4)) return(1);
    return(0);
}

int read_header(FILE* h) {
    char *ctp[8]={  "grayscale", "unknown", "rgb", "indexed",
                    "gray+alpha", "unknown", "rgb+alpha", "unknown" };
    int ct;

    image.width=ngetlong(h); image.height=ngetlong(h);
    image.bitdepth=fgetc(h);
    image.colortype=fgetc(h);
    image.compression=fgetc(h);
    image.filter=fgetc(h);
    image.interlace=fgetc(h);

    printf("PNG Image Header: Size %u x %u\n",image.width,image.height);
    ct=image.colortype; if (ct>7 || ct<0) ct=7;
    printf("BitDepth %d, ColorType %d (%s)\n",image.bitdepth, image.colortype, ctp[ct]);
    printf("Compression %d, Filter %d, Interlace %d\n",image.compression, image.filter, image.interlace);
    ngetlong(h);
}

int options(char* s) {
    char opt_c[]="hvlc";
    char opt_s[][16]={"help", "version", "license", "chunks" };
    int i,xit=0,k;

    if (*s!='-') return(0); // Non inizia per -, ritorno via
    if (s[1]=='-') {
        // Cerca le opzioni del tipo --nomeflag
        for(i=0;i<strlen(opt_c);i++) {
            if (!strcmp(opt_s[i],s+2)) return(do_option(opt_c[i]));
        }
        fprintf(stderr,"Flag unknown: %s\n",s);
    } else {
        // Cerca le opzioni del tipo -<listaflag>
        for (i=1;i<strlen(s);i++) {
            if ((strchr(opt_c,s[i])!=NULL)) {
                k=do_option(s[i]);
                xit=xit || k;
            } else {
                fprintf(stderr,"Flag unknown: -%c\n",s[i]);
            }
        }
        return(xit);
    }
}

int do_option(int o) {
    switch(o) {
        case 'h' : help(); return(1); break;
        case 'v' : version(); return(1); break;
        case 'l' : license(); return(1); break;
        case 'c' : chunklist=1; return(0); break;
    }
    return(0);
}

int help() {
    printf("Png info viewer v 1.0 by Paolo Lucchesi. Usage:\n");
    printf("pngview [-options] [filename]\n");
    printf("-h or --help    Display this page\n");
    printf("-v or --version Display version number\n");
    printf("-l or --license Display software license\n");
    printf("-c or --chunks  Display a list of all chunks\n");
    return(0);
}

int license() {
    printf("Png info viewer v 1.0 by Paolo Lucchesi.\n");
    printf("This program is freeware. Any user can freely use,\n");
    printf("distribute and modify this software without any restriction\n");
    printf("This software is distributed AS IS, with no warranty.\n");
    printf("The author is not responsible for any damage from the use\n");
    printf("of this software.\n");
    return(0);
}

int version() {
    printf("Png info viewer v 1.0\n");
    return(0);
}

int shorthelp() {
    printf("Png info viewer v 1.0. Type pnginfo --help for instructions.\n");
    return(0);
}

