/*
    Png image scrambler
    Questo programma rende illeggibili i dati di una immagine in formato png
    e aggiunge un chunk privato (SrMB) per marcare le immagini modificate

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

unsigned long crc_table[256];

int main(int argc, char** argv) {
    int i,r;
    char nf[128];
    FILE* h;
    char s[128];

    *nf=0;
    for (i=1;i<argc;i++) {
        if (argv[i][0]=='-') {
            if (options(argv[i])) return(0);
        } else {
            if (*nf==0) strncpy(nf,argv[i],127);
        }
    }
    if (*nf==0) { shorthelp(); return(0); }
    make_crc_table();
    r=pngscramble(nf);
    if (!r) filecopy(nf);
    unlink("pngsrmb.tmp");
    return(0);
}

int pngscramble(char* filename) {
    FILE *in,*out;
    void* p;
    long dlen,i,j,os;
    int res;

    printf("Reading file %s\n",filename);
    if ((in=fopen(filename,"rb"))==NULL) {
        printf("File Error!\n"); return(-1); }
    if ((out=fopen("pngsrmb.tmp","wb"))==NULL) {
        printf("Temp File creation Error!\n"); return(-1); }

    if (copy_signature(in,out)) {
        printf("Not a png file!\n"); return(-1); }


    if (copy_chunk(in,out,0)) goto leof;
    if (copy_chunk(in,out,1)) goto leof;
    while (!copy_chunk(in,out,0));
leof:
    fclose(in); fclose(out);
    return(0);
}

int copy_signature(FILE* in, FILE* out) {
    int i,r=0;
    int sig[]={137, 80, 78, 71, 13, 10, 26, 10, -1};

    for (i=0;sig[i]>=0;i++) {
        if (fgetc(in)!=sig[i]) r=-1;
        fputc(sig[i],out);
        if (feof(in)) {r=-1; break; }
    }
    return(r);
}

long ngetlong(FILE* h) {
    long r=0;
    int i;
    for (i=0;i<4;i++) r=r*256+fgetc(h);
    return(r);
}

long nputlong(long n, FILE* h) {
    fputc(((n>>24) & 255),h);
    fputc(((n>>16) & 255),h);
    fputc(((n>>8) & 255),h);
    fputc((n & 255),h);
    return(n);
}    

int copy_chunk(FILE* in, FILE* out, int second) {
    unsigned long len,crc;
    char type[5];
    long i;
    int mask[]={71, 213, 126, 219};
    int mki=0,val;

    len=ngetlong(in); 
    fread(type,1,4,in); type[4]=0; 
    printf("Chunk: %s Lenght: %ld\n",type,len);
    if (second) {
        // Se il secondo chunk e' di tipo SrMB lo tolgo, altrimenti
        // aggiungo un chunk di tipo SrMB
        if (!strncmp(type,"SrMB",4)) {
            ngetlong(in); 
            ngetlong(in);
            return(0);
        }
        nputlong(4,out);
        fputs("SrMBmac?,fcE",out); // SrMB chunk
    }
    nputlong(len,out); fwrite(type,1,4,out);
    if (!strncmp(type,"IDAT",4)) {
        // Chunk di tipo IDAT... ogni byte viene xorrato con una maschera
        // diversa (periodo 4). Il crc viene ricalcolato...
        crc=0xffffffffL;
        for (i=0;i<4;i++) crc = crc_table[(crc ^ type[i]) & 0xff] ^ (crc >> 8);
        for (i=0;i<len;i++) {
            val = fgetc(in) ^ mask[mki];
            mki=(mki+1)%4;
            fputc(val, out);
            crc = crc_table[(crc ^ val) & 0xff] ^ (crc >> 8);
        }
        crc = crc ^ 0xffffffffL;
        ngetlong(in);
    } else {
        // Chunk di tipo diverso... viene copiato pari pari...
        for (i=0;i<len;i++) fputc(fgetc(in),out);
        crc=ngetlong(in);
    }
    nputlong(crc,out);
    if (feof(in)) return(-1);
    if (!strncmp(type,"IEND",4)) return(1);
    return(0);
}

int make_crc_table(void)
{
  unsigned long c;
  int n, k;

  for (n = 0; n < 256; n++) {
    c = (unsigned long) n;
    for (k = 0; k < 8; k++) {
      if (c & 1)
        c = 0xedb88320L ^ (c >> 1);
      else
        c = c >> 1;
    }
    crc_table[n] = c;
  }
  return(0);
}

int filecopy(char* filename) {
    FILE *in,*out;
    char buf[256];
    size_t len;

    if ((in=fopen("pngsrmb.tmp","rb"))==NULL) {
        printf("Temp File reading Error!\n"); return(-1); }
    if ((out=fopen(filename,"wb"))==NULL) {
        printf("File Copying Error!\n"); return(-1); }
    
    for (;;) {
        len=fread(buf,1,256,in);
        if (len<0) break;
        fwrite(buf,1,len,out);
        if (feof(in)) break;
    }
    fclose(in); fclose(out);
    return(0);
}
   

int options(char* s) {
    char opt_c[]="hvl";
    char opt_s[][16]={"help", "version", "license"};
    int i,xit=0,k;

    if (*s!='-') return(0); // Non inizia per -, ritorno via
    if (s[1]=='-') {
        // Cerca le opzioni del tipo --nomeflag
        for(i=0;i<strlen(opt_c);i++) {
            if (!strcmp(opt_s[i],s+2)) return(do_option(opt_c[i]));
        }
        fprintf(stderr,"Flag unknown: %s\n",s);
    } else {
        // Cerca le opzioni del tipo -<listaflag>
        for (i=1;i<strlen(s);i++) {
            if ((strchr(opt_c,s[i])!=NULL)) {
                k=do_option(s[i]);
                xit=xit || k;
            } else {
                fprintf(stderr,"Flag unknown: -%c\n",s[i]);
            }
        }
        return(xit);
    }
}

int do_option(int o) {
    switch(o) {
        case 'h' : help(); return(1); break;
        case 'v' : version(); return(1); break;
        case 'l' : license(); return(1); break;
    }
    return(0);
}

int help() {
    printf("Png image scrambler v 1.0. by Paolo Lucchesi. Usage:");
    printf("pngsrmb [-options] filename\n");
    printf("to scramble or unscramble the 'filename' file\n");
    printf("(per criptare o decriptare il file 'filename')\n");
    printf("-h or --help    Display this page\n");
    printf("-v or --version Display version number\n");
    printf("-l or --license Display software license\n");
    printf("\n");
    return(0);
}

int license() {
    printf("Png image scrambler v 1.0 by Paolo Lucchesi.\n");
    printf("This program is freeware. Any user can freely use,\n");
    printf("distribute and modify this software without any restriction\n");
    printf("This software is distributed AS IS, with no warranty.\n");
    printf("The author is not responsible for any damage from the use\n");
    printf("of this software.\n");
    return(0);
}

int version() {
    printf("Png image scrambler v 1.0\n");
    return(0);
}

int shorthelp() {
    printf("Png image scrambler v 1.0. Type pngsrmb --help for instructions.\n");
    return(0);
}

